indexing

	description: "Base class for all Standard C or POSIX classes."

	author: "Berend de Boer"
	date: "$Date: 2003/01/09 $"
	revision: "$Revision: #3 $"

class

	STDC_BASE

inherit

	ANY

	STDC_CONSTANTS
		export
			{NONE} all
		end

	EPX_EXTERNAL_HELPER
		export
			{NONE} all
		end

	STDC_SECURITY_ACCESSOR


feature -- errno

	errno: STDC_ERRNO is
		once
			create Result
		end


feature {NONE} -- exceptions

	exceptions: EXCEPTIONS is
		once
			create Result
		end

	do_raise (name: STRING) is
		do
			exceptions.raise (name)
		end

	raise_posix_error is
			-- Throws an exception when something that is considered
			-- fatal has occurred.
			-- Exception throwing can be disabled by calling
			-- `security.error_handling.disable_exceptions'
		require
			-- errno_should_be_set: errno.is_not_ok
		do
			if security.error_handling.exceptions_enabled then
				do_raise (errno.message)
			else
				if errno.first_value = 0 then
					errno.set_first
				end
			end
		end

	safe_call (res: INTEGER) is
			-- raise an exception when value = -1
		do
			if res = -1 then
				raise_posix_error
			end
		end


feature {NONE} -- test or set bits in an integer

	flip_bits (bit_field, bits: INTEGER; on: BOOLEAN): INTEGER is
			-- change zero or more bits and return the result
		do
			if on then
				Result := posix_or (bit_field, bits)
			else
				Result := posix_and (bit_field, posix_not (bits))
			end
		ensure
			bits_flipped: (not on) or test_bits (Result, bits)
		end

	test_bits (bit_field, bits: INTEGER): BOOLEAN is
			-- True if given bits are 1 in bit_field
		do
			Result := posix_and (bit_field, bits) = bits
		end


feature {NONE} -- Make bit operations on integers possible

	posix_and (op1, op2: INTEGER): INTEGER is
		external "C"
		end

	posix_not (op: INTEGER): INTEGER is
		external "C"
		end

	posix_or (op1, op2: INTEGER): INTEGER is
		external "C"
		end


end
