indexing

	description:
		"Describes file related security."

	usage: "Inherit from STDC_SECURITY_ACCESSOR."

	author: "Berend de Boer"
	date: "$Date: 2003/01/09 $"
	revision: "$Revision: #3 $"


class

	STDC_SECURITY_FILES


inherit

	STDC_SECURITY_ASPECT


creation {STDC_SECURITY}

	make


feature {NONE} -- Initialization

	make is
			-- Allow everything.
		do
			max_open_files := Max_Int
		ensure
			no_files_open: open_files = 0
		end


feature -- Parameters

	max_open_files: INTEGER
			-- Max files that may be open.

	open_files: INTEGER is
			-- The files that are open.
		external "C"
		alias "posix_open_files"
		end


feature -- set parameters

	set_max_open_files (value: INTEGER) is
			-- Change the number of files that maybe open at a given
			-- time.
		require
			value_not_less_than_open_files: value >= open_files
		do
			max_open_files := value
			-- Check if we keep within the limits by hitting
			-- `check_max_open_files' with the current number of
			-- `open_files'.
			decrease_open_files
			increase_open_files
		end


feature {STDC_SECURITY_ACCESSOR} -- Count resource usage

	decrease_open_files is
		external "C"
		alias "posix_decrease_open_files"
		ensure
			open_files_decreased: open_files = old open_files - 1
		end

	increase_open_files is
		do
			check_max_open_files
			posix_increase_open_files
		ensure
			open_files_increased: open_files = old open_files + 1
		end


feature {NONE} -- Implementation

	posix_increase_open_files is
		external "C"
		ensure
			open_files_increased: open_files = old open_files + 1
		end


feature -- Checks

	check_max_open_files is
			-- Raise exception if incrementing the number of open files
			-- will exceeded `max_open_files'.
		do
			if open_files >= max_open_files then
				raise_security_error ("Maximum allowed open files exceeded.")
			end
		end


invariant

	open_files_not_negative: open_files >= 0
	max_open_files_positive: max_open_files > 0
	open_files_within_limit: open_files <= max_open_files

end
