indexing

	description: "Class that describes service names, i.e. an entry %
	%in /etc/services."

	author: "Berend de Boer"
	date: "$Date: 2003/01/09 $"
	revision: "$Revision: #3 $"


class

	SUS_SERVICE


inherit

	SUS_BASE

	SAPI_NETDB

	SAPI_IN


creation

	make_from_name,
	make_from_port


feature -- creation

	make_from_name (a_name, a_protocol: STRING) is
			-- Find service with `a_name' and optional `a_protocol' or raise
			-- exception.
		require
			name_not_empty: a_name /= Void and then not a_name.is_empty
		local
			p: POINTER
		do
			p := posix_getservbyname (
				sh.string_to_pointer (a_name),
				sh.string_to_pointer (a_protocol))
			if p = default_pointer then
				-- service with this name does not exist
				raise_posix_error
			else
				set_members_from_struct (p, 0, a_protocol)
			end
		end

	make_from_port (a_port: INTEGER; a_protocol: STRING) is
			-- Initialize service from given a_port
		require
			valid_port: a_port >= 0 and a_port <= 65535
			-- make sure you have a `a_protocol' if necessary!
		local
			nport: INTEGER
			p: POINTER
		do
			nport := posix_htons (a_port)
			p := posix_getservbyport (nport, sh.string_to_pointer (a_protocol))
			set_members_from_struct (p, a_port, a_protocol)
		end


feature -- public state, all data in host byte order

	port: INTEGER
			-- port number

	name: STRING
			-- official service name

	aliases: ARRAY [STRING]
			-- alias list

	protocol: STRING
			-- protocol to use (udp/tcp)

	protocol_type: INTEGER
			-- SOCK_STREAM or SOCK_DGRAM


feature {NONE} -- set

	set_members_from_struct (p: POINTER; a_port: INTEGER; a_protocol: STRING) is
			-- Initialize attributes of object from struct in `p'. If p =
			-- default_pointer we assume service does not exist in
			-- /etc/services and give some dummy values.
		require
			valid_port: a_port >= 0 and a_port <= 65535
		do
			if p = default_pointer then
				-- not found, guess?
				port := a_port
				name := "unknown"
				protocol := a_protocol
				create aliases.make (0, -1)
			else
				port := posix_ntohs (posix_servent_s_port (p))
				name := sh.pointer_to_string (posix_servent_s_name (p))
				protocol := sh.pointer_to_string (posix_servent_s_proto (p))
				aliases := ah.pointer_to_string_array (posix_servent_s_aliases (p))
			end
			if protocol /= Void then
				protocol.to_lower
				if protocol.is_equal (once_tcp) then
					protocol_type := SOCK_STREAM
				elseif protocol.is_equal (once_udp) then
					protocol_type := SOCK_DGRAM
				else
					-- guess?, raise error?
					protocol_type := SOCK_STREAM
				end
			else
				-- just give it a value, or perhaps invent a NULL value
				-- for `protocol_type', perhaps better.
				protocol_type := SOCK_STREAM
			end
		end


feature {NONE} -- Once strings

	once_tcp: STRING is "tcp"
	once_udp: STRING is "udp"

invariant

	name_void_or_not_empty: name = Void or else not name.is_empty
	valid_port: port >= 0 and port <= 65535
	valid_protocol:
		(protocol = Void or else protocol.is_empty) or else
		(protocol.is_equal (once_tcp) or protocol.is_equal (once_udp))
	valid_protocol_type:
		protocol_type = SOCK_STREAM or else
		protocol_type = SOCK_DGRAM
	valid_aliases: aliases /= Void

end
